#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){

	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	ofEnableSmoothing();
	
	allowMove = true;
	
	// 2 output channels,
	// 0 input channels
	// 22050 samples per second
	// 256 samples per buffer
	// 4 num buffers (latency)
	
	sampleRate 			= 44100;
	phase 				= 0;
	phaseAdder 			= 0.0f;
	phaseAdderTarget 	= 0.0f;
	volume				= 0.1f;
	bNoise 				= false;
	lAudio = new float[256];
	rAudio = new float[256];
	ofSoundStreamSetup(2,0,this, sampleRate,256, 4);

	
}

//--------------------------------------------------------------s
void testApp::update(){

	for(int i=0; i<NBALLS; i++)
	{
			
		if (i==0) {
			if(allowMove==true){
				myBall[i].moveTo(prevx, prevy);
				int width = ofGetWidth();
				float pct =  ((float)myBall[i].xpos/(float)myBall[i].ypos/width)*100;
				phase = pct;
				pan = (float)myBall[i].ypos / (float)width;
				float height = (float)ofGetHeight();
				float heightPct = ((height-myBall[i].xpos) / height);
				targetFrequency = 1000.0f * heightPct;
				phaseAdderTarget = (targetFrequency / pct) * TWO_PI;
			}
		} else {
			if(allowMove==true){
				myBall[i].moveTo(myBall[i-1].xpos, myBall[i-1].ypos);
				int width = ofGetWidth();
				float pct =  ((float)myBall[i].xpos/(float)myBall[i].ypos/width)*100;
				phase = pct;
				pan = (float)myBall[i].ypos / (float)width;
				float height = (float)ofGetHeight();
				float heightPct = ((height-myBall[i].xpos) / height);
				targetFrequency = 1000.0f * heightPct;
				phaseAdderTarget = (targetFrequency / pct) * TWO_PI;
			}
		}
	
	}
	
	
	for(int i=0; i<NBALLS; i++)
	{
		
		if (i==0) {
			if(allowMove==false){

				int width = ofGetWidth();
				float widthPct = ((width-myBall[i].ypos) / width);
				int height = ofGetHeight();
				float heightPct = ((height-myBall[i].xpos) / height);
				int randX = ofRandom(0,height);
				int randY = ofRandom(0,width);
				//myBall[i].moveToRandom(randX, randY);
				myBall[i].update();
				float pct =  ((float)myBall[i].xpos/(float)myBall[i].ypos/width)*100;
				phase = pct;
				pan = (float)myBall[i].ypos / (float)width;
				targetFrequency = 4000.0f * heightPct;
				phaseAdderTarget = (targetFrequency / phase) * TWO_PI;
			}
		} else {
			if(allowMove==false){
				int width = ofGetWidth();
				float widthPct = ((width-myBall[i].ypos) / width);
				int height = ofGetHeight();
				float heightPct = ((height-myBall[i].xpos) / height);
				int randX = ofRandom(0,height);
				int randY = ofRandom(0,width);
				//myBall[i-1].moveToRandom(randX, randY);
				myBall[i-1].update();
				float pct =  ((float)myBall[i-1].xpos/(float)myBall[i-1].ypos/width)*100;
				phase = pct;
				pan = (float)myBall[i].ypos / (float)width;
				targetFrequency = 4000.0f * heightPct;
				phaseAdderTarget = (targetFrequency / phase) * TWO_PI;
			}
		}
		
	}	
		
	
	

}

//--------------------------------------------------------------
void testApp::draw(){
	char reportString[255];
	//sprintf(reportString, "phase: (%f) modify with -/+ keys\npan: (%f)\nsynthesis: %s", phase, pan, bNoise ? "noise" : "sine wave");
	
	ofDrawBitmapString(reportString,80,380);
	
	for(int i=0; i<NBALLS; i++)
	{
		myBall[i].draw();
		if (i!=0) {
			ofLine(myBall[i].xpos, myBall[i].ypos, myBall[i-1].xpos, myBall[i-1].ypos);
		}
	}

}

//--------------------------------------------------------------
void testApp::keyPressed(int key){
	if (key == 'r'){
		allowMove=false;
	}
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){
	if (key == 'r'){
		allowMove=true;
	}
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

	for(int i=0; i<NBALLS; i++)
	{
		myBall[i].update();
	}
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	for(int i=0; i<NBALLS; i++)
	{
		myBall[i].update();
	}

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	
	prevx = x;
	prevy = y;

}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){


}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void testApp::audioRequested 	(float * output, int bufferSize, int nChannels){
	//pan = 0.5f;
	float leftScale = 1 - pan;
	float rightScale = pan;
	
	// sin (n) seems to have trouble when n is very large, so we
	// keep phase in the range of 0-TWO_PI like this:
	while (phase > TWO_PI){
		phase -= TWO_PI;
	}
	
	if ( bNoise == true){
		// ---------------------- noise --------------
		for (int i = 0; i < bufferSize; i++){
			lAudio[i] = output[i*nChannels    ] = ofRandomf() * volume * leftScale;
			rAudio[i] = output[i*nChannels + 1] = ofRandomf() * volume * rightScale;
		}
	} else {
		phaseAdder = 0.95f * phaseAdder + 0.05f * phaseAdderTarget;
		for (int i = 0; i < bufferSize; i++){
			phase += phaseAdder;
			float sample = sin(phase);
			lAudio[i] = output[i*nChannels    ] = sample * volume * leftScale;
			rAudio[i] = output[i*nChannels + 1] = sample * volume * rightScale;
		}
	}
	
}

